// Copyright (C) 2019 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
// Qt-Security score:significant reason:default

#ifndef QTHTTPSERVERGLOBAL_H
#define QTHTTPSERVERGLOBAL_H

#include <QtCore/qglobal.h>
#include <QtHttpServer/qthttpserverexports.h>

#endif // QTHTTPSERVERGLOBAL_H

